package Package1;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.TreeMap;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingWorker;
import javax.swing.SwingWorker.StateValue;
import javax.swing.WindowConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XmlStatsGraphicsThread extends JFrame {

    TreeMap<Integer, Integer> rhythmMap = new TreeMap<>();
    int[] pitchStats = {0, 0, 0, 0, 0, 0, 0};
    MyPanel panel;
    JButton button;
    SwingWorker worker;

    private void calculateStats() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setIgnoringElementContentWhitespace(true);
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            File file = new File("gounod_ave_maria.xml");
            Document doc = builder.parse(file);

            // altezze
            NodeList pitchNodeList = doc.getElementsByTagName("pitch");
            // Non è possibile usare un foreach perché NodeList non implementa l'interfaccia Iterable
            Node currentNode = null;
            for (int i = 0; i < pitchNodeList.getLength() && (currentNode = pitchNodeList.item(i)) != null; i++) {
                Element pitchElem = (Element) currentNode;
                if (!pitchElem.getAttribute("step").equals("")) {
                    char currentPitch = pitchElem.getAttribute("step").toUpperCase().charAt(0);
                    int arrayIndex = (int) currentPitch - 'A';
                    pitchStats[arrayIndex]++;
                }
            }

            // durate
            NodeList rhythmNodeList = doc.getElementsByTagName("duration");
            for (int i = 0; i < rhythmNodeList.getLength() && (currentNode = rhythmNodeList.item(i)) != null; i++) {
                Element rhythmElem = (Element) currentNode;
                if (rhythmElem.getAttributeNode("den") != null) {
                    if (rhythmElem.getAttributeNode("num") != null) {
                        int currentNum = Integer.parseInt(rhythmElem.getAttributeNode("num").getTextContent());
                        if (currentNum != 1) {
                            continue;
                        }
                    }
                    int currentDen = Integer.parseInt(rhythmElem.getAttributeNode("den").getTextContent());
                    int newCounterValue = 0;
                    if (rhythmMap.containsKey(currentDen)) {
                        newCounterValue = rhythmMap.get(currentDen);
                    }
                    rhythmMap.put(currentDen, ++newCounterValue);
                }
            }

        } catch (ParserConfigurationException | SAXException | IOException e) {
            System.out.println("Errore nell'elaborazione del file");
            System.exit(1);
        }
    }

    public class MyPanel extends JPanel {

        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);

            if (worker != null && !worker.isDone()) {
                g.setFont(new Font(null, Font.PLAIN, 30));
                if (worker.getState() == StateValue.PENDING)
                    g.drawString("Premere start per iniziare", 10, g.getFontMetrics().getHeight());
                else
                    g.drawString("Calcolo in corso...", 10, g.getFontMetrics().getHeight());
            } else {
                int width = getSize().width;
                int height = getSize().height;
                int fontHeight = g.getFontMetrics().getHeight();
                int fontDescent = g.getFontMetrics().getDescent();
                g.setColor(Color.BLACK);
                g.drawLine(0, height / 2, width, height / 2);
                g.drawString("Occorrenze note", 2, height / 2 - 3 * fontDescent);
                g.drawString("Occorrenze durate", 2, height / 2 + fontHeight);

                float widthGraph1 = (float) width / pitchStats.length;
                int max1 = 0;
                for (int i = 0; i < pitchStats.length; i++) {
                    max1 = Math.max(max1, pitchStats[i]);
                }
                float heightMult1 = height / 3f / max1;
                float colorMult1 = 150 / pitchStats.length;
                for (int i = 0; i < pitchStats.length; i++) {
                    g.setColor(new Color(0, 0, (int) (100 + i * colorMult1)));
                    g.drawString((char) ('A' + i) + ": " + pitchStats[i],
                            (int) (i * widthGraph1 + 2), fontHeight - fontDescent);
                    g.fillRect((int) (i * widthGraph1) + 2, fontHeight,
                            (int) widthGraph1 - 4, (int) (pitchStats[i] * heightMult1));
                }

                float widthGraph2 = (float) width / rhythmMap.keySet().size();
                int max2 = 0;
                for (int key : rhythmMap.keySet()) {
                    max2 = Math.max(max2, rhythmMap.get(key));
                }
                float heightMult2 = height / 3f / max2;
                float colorMult2 = 150 / pitchStats.length;
                int i = 0;
                for (int key : rhythmMap.keySet()) {
                    g.setColor(new Color(0, (int) (100 + i * colorMult2), 0));
                    g.drawString("1/" + key + ": " + rhythmMap.get(key),
                            (int) (i * widthGraph2 + 2), height - fontDescent);
                    g.fillRect((int) (i * widthGraph2) + 2, height - fontHeight - (int) (rhythmMap.get(key) * heightMult2),
                            (int) widthGraph2 - 4, (int) (rhythmMap.get(key) * heightMult2));
                    i++;
                }
            }
        }
    }

    public XmlStatsGraphicsThread(String title, int width, int height) {
        super(title);
        setLayout(new FlowLayout());
        panel = new MyPanel();
        panel.setBackground(Color.WHITE);
        panel.setPreferredSize(new Dimension(width, 3 * height / 4));
        add(panel);
        button = new JButton("Start");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (button.getText().equals("Start")) {
                    worker.execute();
                    button.setText("Stop");
                } else
                    worker.cancel(true);
            }
        });
        add(button);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setSize(width, height);
        pack();
        setVisible(true);
        worker = new SwingWorker<Void, Void>() {
            @Override
            public Void doInBackground() {
                calculateStats();
                return null;
            }
        };
        worker.addPropertyChangeListener(new PropertyChangeListener() {
            @Override
            public void propertyChange(final PropertyChangeEvent event) {
                switch (event.getPropertyName()) {
                    case "progress":
                        System.out.println((Integer)event.getNewValue());
                        break;
                    case "state":
                        switch ((StateValue) event.getNewValue()) {
                            case PENDING:  // Stato iniziale
                                System.out.println("PENDING");
                                panel.repaint();
                                break;
                            case DONE: // Task concluso
                                System.out.println("DONE");
                                panel.repaint();
                                button.setText("Done");
                                button.setEnabled(false);
                                break;
                            case STARTED: // Prima dell’esecuzione di doInBackground
                                System.out.println("STARTED");
                                panel.repaint();
                                break;
                        }
                        break;
                }
            }

        });
    }

    public static void main(String[] args) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new XmlStatsGraphicsThread("Statistiche IEEE 1599", 500, 400);
            }
        });
    }
}
