package audio;

public class WavHeader {

    public static byte[] returnHeader(int numSamples, int sampleRate, int numChannels, int bitsPerSample) {

        byte[] header = new byte[44];

        header[0] = 'R';
        header[1] = 'I';
        header[2] = 'F';
        header[3] = 'F';

        int audioSamplesByteSize = numSamples * numChannels * bitsPerSample / 8;
        int chunkSize = (int) (36 + audioSamplesByteSize);
        // bit shifting e maschera di bit
        header[4] = (byte) (chunkSize & 0xff);
        header[5] = (byte) ((chunkSize >> 8) & 0xff);
        header[6] = (byte) ((chunkSize >> 16) & 0xff);
        header[7] = (byte) ((chunkSize >> 24) & 0xff);

        header[8] = 'W';
        header[9] = 'A';
        header[10] = 'V';
        header[11] = 'E';

        // Sub-chunk fmt ----------------------------------------------------
        // header
        header[12] = 'f';
        header[13] = 'm';
        header[14] = 't';
        header[15] = ' ';

        // subchunk size
        header[16] = 16; // little endian
        header[17] = 0;
        header[18] = 0;
        header[19] = 0;

        // Audio format: PCM
        header[20] = 1; // little endian
        header[21] = 0;

        // Number of channels
        header[22] = (byte) (numChannels & 0xff); // little endian
        header[23] = (byte) ((numChannels >> 8) & 0xff);

        // Sample rate
        header[24] = (byte) (sampleRate & 0xff);
        header[25] = (byte) ((sampleRate >> 8) & 0xff);
        header[26] = (byte) ((sampleRate >> 16) & 0xff);
        header[27] = (byte) ((sampleRate >> 24) & 0xff);

        // Byte rate
        int byteRate = sampleRate * numChannels * bitsPerSample / 8;
        header[28] = (byte) (byteRate & 0xff);
        header[29] = (byte) ((byteRate >> 8) & 0xff);
        header[30] = (byte) ((byteRate >> 16) & 0xff);
        header[31] = (byte) ((byteRate >> 24) & 0xff);

        // Block align
        int blockAlign = numChannels * bitsPerSample / 8;
        header[32] = (byte) (blockAlign & 0xff);
        header[33] = (byte) ((blockAlign >> 8) & 0xff);

        // Bit depth
        header[34] = (byte) (bitsPerSample & 0xff);
        header[35] = (byte) ((bitsPerSample >> 8) & 0xff);

        // Sub-chunk data ----------------------------------------------------
        // header
        header[36] = 'd';
        header[37] = 'a';
        header[38] = 't';
        header[39] = 'a';

        // subchunk size
        header[40] = (byte) (audioSamplesByteSize & 0xff);
        header[41] = (byte) ((audioSamplesByteSize >> 8) & 0xff);
        header[42] = (byte) ((audioSamplesByteSize >> 16) & 0xff);
        header[43] = (byte) ((audioSamplesByteSize >> 24) & 0xff);

        return header;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        byte[] header = returnHeader(80, 8000, 2, 16);
        for (int i = 0; i < header.length; i++)
          System.out.println(String.format("0x%02X", header[i]) + " ");
    }
    
}
