package jSyn;

import com.jsyn.JSyn;
import com.jsyn.Synthesizer;
import com.jsyn.data.FloatSample;
import com.jsyn.unitgen.*;
import com.jsyn.util.SampleLoader;
import java.io.File;
import java.io.IOException;
import jm.music.tools.ca.CellularAutomata;

public class GranularCA {

    Synthesizer synth;
    SampleGrainFarm sampleGrainFarm;
    ContinuousRamp ramp;
    LineOut lineOut;
    final int NUM_GRAINS = 50;
    boolean[][] cells = new boolean[10][10];
    int liveCellCounter;

    private GranularCA() throws IOException, InterruptedException {
        synth = JSyn.createSynthesizer();
        synth.start();
        synth.add(lineOut = new LineOut());

        synth.add(sampleGrainFarm = new SampleGrainFarm());
        // Load a sample that we want to "granulate" from a file.
        File sampleFile = new File("pini.wav");
        FloatSample sample = SampleLoader.loadFloatSample(sampleFile);
        sampleGrainFarm.setSample(sample);
        sampleGrainFarm.allocate(NUM_GRAINS);
        // Use a ramp to move smoothly within the file.
        synth.add(ramp = new ContinuousRamp());
        ramp.output.connect(sampleGrainFarm.position);

        sampleGrainFarm.output.connect(0, lineOut.input, 0);
        sampleGrainFarm.output.connect(0, lineOut.input, 1);
        lineOut.start();

        CellularAutomata CA = new CellularAutomata(cells.length, cells[0].length, 30, false);
        // do the iterations
        while (true) {
            CA.evolve();
            cells = CA.getAllStates();
            //counts the number of live cells in the current iteration
            liveCellCounter = 0;
            for (int k = 0; k < cells.length; k++)
                for (int j = 0; j < cells[0].length; j++)
                    if (CA.getState(k, j) == true)
                        liveCellCounter++;

            sampleGrainFarm.rate.set((float)liveCellCounter / cells.length / 2);
            sampleGrainFarm.duration.set((float)liveCellCounter / cells.length);

            CA.print();
            // if the automaton is completely dead, try to "resurrect" some cells
            if (liveCellCounter == 0)
                CA.setGrid(2);
        }
    }

    public static void main(String[] args) throws IOException, InterruptedException {
        new GranularCA();
    }
}
